//============================================================================
//
// SPI to RGB initiaize routine for CFAF480640 /  ILI9806E
//
//---------------------------------------------------------------------------
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
#include <SPI.h>

#include "ILI9806E_Init.h"

//----------------------------------------------------------------------------
void LCD_Write_Command(uint8_t command)
  {
  //Select the LCD
    delay(10);


  CLR_DISP_CS;
  uint16_t
    address_and_command;
  //Select the Command register:
  address_and_command=0x0000;
  //Set the low 8 bits from the caller
  address_and_command|=command;
  // Send the data
  uint16_t
    mask;
  for(mask=0x0100;0!=mask;mask>>=1)
     {
     //Set or clear the data bit = MOSI
     if(0!=(mask&address_and_command))
       {
       SET_MOSI_LCD;
       }
     else
       {
       CLR_MOSI_LCD;
       }
     //Toggle the clock
     SET_SCK_LCD;
     delayMicroseconds(1);
     CLR_SCK_LCD;
     delayMicroseconds(1);
     }
  //De-select the LCD
  SET_DISP_CS;
  }
//----------------------------------------------------------------------------
void LCD_Write_Data(uint8_t data)
  {
  //Select the LCD
  CLR_DISP_CS;
  uint16_t
    address_and_data;
  //Select the Data register;
  address_and_data=0x0100;
  //Set the low 8 bits from the caller
  address_and_data|=data;
  // Send the data
  uint16_t
    mask;
  for(mask=0x0100;0!=mask;mask>>=1)
     {
     //Set or clear the data bit = MOSI
     if(0!=(mask&address_and_data))
       {
       SET_MOSI_LCD;
       }
     else
       {
       CLR_MOSI_LCD;
       }
     //Toggle the clock
     SET_SCK_LCD;
     delayMicroseconds(1);
     CLR_SCK_LCD;
     delayMicroseconds(1);
     }
  //De-select the LCD
  SET_DISP_CS;
  }
//----------------------------------------------------------------------------
void ILI9806E_LCD_Init(void)
  {
   CLR_DISP_RST;
   delay(20);
   SET_DISP_RST;
   delay(120);

   //***************************************************************//LCD SETING
LCD_Write_Command(0xFF);        // Change to Page 1 CMD 
LCD_Write_Data(0xff); 
LCD_Write_Data(0x98); 
LCD_Write_Data(0x06); 
LCD_Write_Data(0x04); 
LCD_Write_Data(0x01); 

LCD_Write_Command(0x08); //Output    SDA 
LCD_Write_Data(0x10); //10

LCD_Write_Command(0x20);//set DE/VSYNC mode   
LCD_Write_Data(0x00);     //01 VSYNC MODE

LCD_Write_Command(0x21); //DE = 1 Active 
LCD_Write_Data(0x01);//01 

LCD_Write_Command(0x23); 
LCD_Write_Data(0x02);
 
LCD_Write_Command(0x25); 
LCD_Write_Data(0x0A);

LCD_Write_Command(0x26); 
LCD_Write_Data(0x14);

LCD_Write_Command(0x27); 
LCD_Write_Data(0x14);

LCD_Write_Command(0x28); 
LCD_Write_Data(0x00);


LCD_Write_Command(0x30);//Resolution setting 480 X 640 
LCD_Write_Data(0x03); //03
 
LCD_Write_Command(0x31); //Inversion setting 
LCD_Write_Data(0x00); //02-2dot
 
LCD_Write_Command(0x40); //BT DDVDH DDVDL
LCD_Write_Data(0x11); //10,14,18 00 2XVCI
 
LCD_Write_Command(0x41);  
LCD_Write_Data(0x44);  //avdd +5.2v,avee-5.2v 33
 
LCD_Write_Command(0x42);        
LCD_Write_Data(0x01); //VGL=DDVDH+VCIP -DDVDL,VGH=2DDVDL-VCIP  
 
LCD_Write_Command(0x43);       
LCD_Write_Data(0x89); //SET VGH clamp level +15v
 
LCD_Write_Command(0x44);       
LCD_Write_Data(0x89); //SET VGL clamp level -10v

LCD_Write_Command(0x46);    
LCD_Write_Data(0x34);

LCD_Write_Command(0x50);//VREG1 for positive Gamma 
LCD_Write_Data(0x90); //A8
 
LCD_Write_Command(0x51);//VREG2 for negative Gamma 
LCD_Write_Data(0x90); //A8
 
LCD_Write_Command(0x52);//VCOM 
LCD_Write_Data(0x00); 
 
LCD_Write_Command(0x53); //Forward Flicker
LCD_Write_Data(0x55); //VCOM 
 
LCD_Write_Command(0x54); //VCOM 
LCD_Write_Data(0x00); 

LCD_Write_Command(0x55); //Backward Flicker
LCD_Write_Data(0x55); //VCOM 

LCD_Write_Command(0x60);       
LCD_Write_Data(0x07); 
 
LCD_Write_Command(0x61);        
LCD_Write_Data(0x04); 
 
LCD_Write_Command(0x62);      
LCD_Write_Data(0x08); 
 
LCD_Write_Command(0x63);      
LCD_Write_Data(0x04); 

LCD_Write_Command(0xA0);  //Positive Gamma 
LCD_Write_Data(0x00); 
LCD_Write_Command(0xA1);        // 
LCD_Write_Data(0x09); 
LCD_Write_Command(0xA2);        // 
LCD_Write_Data(0x11); 
LCD_Write_Command(0xA3);        // 
LCD_Write_Data(0x0B); 
LCD_Write_Command(0xA4);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0xA5);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0xA6);        // 
LCD_Write_Data(0x07); 
LCD_Write_Command(0xA7);        // 
LCD_Write_Data(0x06); 
LCD_Write_Command(0xA8);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0xA9);        // 
LCD_Write_Data(0x0B); 
LCD_Write_Command(0xAA);        // 
LCD_Write_Data(0x1C); 
LCD_Write_Command(0xAB);        // 
LCD_Write_Data(0x14); 
LCD_Write_Command(0xAC);        // 
LCD_Write_Data(0x1A); 
LCD_Write_Command(0xAD);        // 
LCD_Write_Data(0x1C); 
LCD_Write_Command(0xAE);        // 
LCD_Write_Data(0x1E); 
LCD_Write_Command(0xAF);        // 
LCD_Write_Data(0x0E); 
 
LCD_Write_Command(0xC0);        //Negative Gamma 
LCD_Write_Data(0x00); 
LCD_Write_Command(0xC1);        // 
LCD_Write_Data(0x09); 
LCD_Write_Command(0xC2);        // 
LCD_Write_Data(0x11); 
LCD_Write_Command(0xC3);        // 
LCD_Write_Data(0x0B); 
LCD_Write_Command(0xC4);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0xC5);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0xC6);        // 
LCD_Write_Data(0x07); 
LCD_Write_Command(0xC7);        // 
LCD_Write_Data(0x06); 
LCD_Write_Command(0xC8);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0xC9);        // 
LCD_Write_Data(0x0B); 
LCD_Write_Command(0xCA);        // 
LCD_Write_Data(0x1C); 
LCD_Write_Command(0xCB);        // 
LCD_Write_Data(0x14); 
LCD_Write_Command(0xCC);        // 
LCD_Write_Data(0x1A); 
LCD_Write_Command(0xCD);        // 
LCD_Write_Data(0x1C); 
LCD_Write_Command(0xCE);        // 
LCD_Write_Data(0x1E); 
LCD_Write_Command(0xCF);        // 
LCD_Write_Data(0x0E); 


LCD_Write_Command(0xFF); // Change to Page 6 CMD for GIP timing   
LCD_Write_Data(0xFF); 
LCD_Write_Data(0x98); 
LCD_Write_Data(0x06); 
LCD_Write_Data(0x04); 
LCD_Write_Data(0x06);

LCD_Write_Command(0x00);        // 
LCD_Write_Data(0x21); 
LCD_Write_Command(0x01);        // 
LCD_Write_Data(0x09); 
LCD_Write_Command(0x02);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x03);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x04);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x05);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x06);        // 
LCD_Write_Data(0x98); 
LCD_Write_Command(0x07);        // 
LCD_Write_Data(0x05); 
LCD_Write_Command(0x08);        // 
LCD_Write_Data(0x02); 
LCD_Write_Command(0x09);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x0A);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x0B);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x0C);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x0D);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x0E);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x0F);        // 
LCD_Write_Data(0x00);

LCD_Write_Command(0x10);        // 
LCD_Write_Data(0xE0); 
LCD_Write_Command(0x11);        // 
LCD_Write_Data(0xE0); 
LCD_Write_Command(0x12);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x13);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x14);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x15);        // 
LCD_Write_Data(0x43); 
LCD_Write_Command(0x16);        // 
LCD_Write_Data(0x0B); 
LCD_Write_Command(0x17);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x18);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x19);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x1A);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x1B);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x1C);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x1D);        // 
LCD_Write_Data(0x00); 

LCD_Write_Command(0x20);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x21);        // 
LCD_Write_Data(0x23); 
LCD_Write_Command(0x22);        // 
LCD_Write_Data(0x45); 
LCD_Write_Command(0x23);        // 
LCD_Write_Data(0x67); 
LCD_Write_Command(0x24);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x25);        // 
LCD_Write_Data(0x23); 
LCD_Write_Command(0x26);        // 
LCD_Write_Data(0x45); 
LCD_Write_Command(0x27);        // 
LCD_Write_Data(0x67); 

LCD_Write_Command(0x30);        // 
LCD_Write_Data(0x01); 
LCD_Write_Command(0x31);        // 
LCD_Write_Data(0x11); 
LCD_Write_Command(0x32);        // 
LCD_Write_Data(0x00); 
LCD_Write_Command(0x33);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x34);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x35);        // 
LCD_Write_Data(0xcb); 
LCD_Write_Command(0x36);        // 
LCD_Write_Data(0xda); 
LCD_Write_Command(0x37);        // 
LCD_Write_Data(0xAD); 
LCD_Write_Command(0x38);        // 
LCD_Write_Data(0xbc); 
LCD_Write_Command(0x39);        // 
LCD_Write_Data(0x67); 
LCD_Write_Command(0x3A);        // 
LCD_Write_Data(0x76); 
LCD_Write_Command(0x3B);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x3C);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x3D);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x3E);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x3F);        // 
LCD_Write_Data(0x22); 
LCD_Write_Command(0x40);        // 
LCD_Write_Data(0x22); 

LCD_Write_Command(0xFF);        // Change to Page 7 CMD for GIP timing   
LCD_Write_Data(0xFF); 
LCD_Write_Data(0x98); 
LCD_Write_Data(0x06); 
LCD_Write_Data(0x04); 
LCD_Write_Data(0x07); 
 
LCD_Write_Command(0x18);        // 
LCD_Write_Data(0x1d); 

LCD_Write_Command(0x02);        // 
LCD_Write_Data(0x77); 

LCD_Write_Command(0xE1);        // 
LCD_Write_Data(0x79); 

LCD_Write_Command(0xFF);        // Change to Page 0 CMD for Normal command 
LCD_Write_Data(0xFF); 
LCD_Write_Data(0x98); 
LCD_Write_Data(0x06); 
LCD_Write_Data(0x04); 
LCD_Write_Data(0x00);

LCD_Write_Command(0x36);    
LCD_Write_Data(0x01);

LCD_Write_Command(0x3A);      
LCD_Write_Data(0x50); //16-BIT

// Exit sleep
LCD_Write_Command(0x11);
delay(120);
LCD_Write_Command(0x29);
delay(25);


}
  
//============================================================================
void Exit_Sleep()
{
// Exit sleep
LCD_Write_Command(0x11);
delay(120);
LCD_Write_Command(0x29);
}

//============================================================================
void Enter_Sleep()
{
// Enter sleep
LCD_Write_Command(0x28);
delay(10);
LCD_Write_Command(0x10);
}
